"""
cursor
===============================================================================
"""

import wx
from wx.lib.embeddedimage import PyEmbeddedImage

cursors = {}

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgPAwMAA"
    "AACAgID///9hUlqkAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAAQUlEQVQoz2NgoAoQRBcQMkQTYFZCE2FUQhcxQhcRUlJCN0QB3RAmNHsMGNCUCDCg"
    "KwE61wBNQBhNE4ORAsMoAAIAYm8DsPm51CwAAAAldEVYdGRhdGU6Y3JlYXRlADIwMTctMDgt"
    "MThUMTQ6MTc6MzYrMDI6MDDbBp0NAAAAJXRFWHRkYXRlOm1vZGlmeQAyMDE3LTA4LTE4VDE0"
    "OjE4OjE1KzAyOjAwKJ1j3AAAAABJRU5ErkJggg==").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['Default'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAACXBIWXMAAAsTAAALEwEAmpwY"
    "AAAArUlEQVRYhe2VQQ7DIAwE130ZX/PPlpdtDy1VCdBTsXPAEpckkmcnljFJyKxHavcDcFuA"
    "WmvYZE4BSEb1ByR1h6QAiKSu73acwQBJSLIwC7P07+UUYqEz0NKbmcIsrNJHWfgYaOkvcNst"
    "WLuMmvbhg8Xzf9V0D5hZ/CIiCXdvA2AA4O4K+wXAK/l3w1LKMBfbAH5pD5mBVZPdBu55HUdW"
    "N4QZlW7gAByAA3AA0gGeYlZtKH9TdOAAAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['Draw'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgAgMAAAAOFJJnAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAADFBMVEUBAgMAAACA"
    "gID///9fpk1YAAAAAXRSTlMAQObYZgAAAAFiS0dEAxEMTPIAAAAHdElNRQfhCBIOEg85efYa"
    "AAAARElEQVQY02NggAMHDEYDlBZLRRdhcIMxPBOgDMYwKIMpFCYUCjNpKky1CExEQQAmxAJ3"
    "gCjcAVBVjAwCaAy4FEIxbQEAg2EG96jiK74AAAAldEVYdGRhdGU6Y3JlYXRlADIwMTctMDgt"
    "MThUMTQ6MTc6MzYrMDI6MDDbBp0NAAAAJXRFWHRkYXRlOm1vZGlmeQAyMDE3LTA4LTE4VDE0"
    "OjE4OjE1KzAyOjAwKJ1j3AAAAABJRU5ErkJggg==").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['DrawEllipse'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAACXBIWXMAAAsTAAALEwEAmpwY"
    "AAAA5ElEQVRYhe2WWQ6DMAxEx1XvBUdLbmafzP0oqbKYbhj8k5EiRBTwy3hEIFVFpG6h1ScA"
    "gPsvi0WkDgwty3IYwHSgK1TPUTVc0msCMLPHu7+TqjaDmRWAMrP288zc3PfP/jMGB7YiZLhA"
    "AFREtGrHYTUAIqIpJQBASsnKQp0BFzUAZfdEpL0LHok31fd++zS/rl69/piBsvsOzsqCq6gc"
    "RsX2YcHOvJfM7wARXXZENi3IOZcAEADknPWyFgDPndcF13UdcnEawDvbL8nAXpGzHQj/HwgH"
    "aEIYoXAHJsAEmAATIBzgASzTKthtoxiTAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['DrawEnd'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgAgMAAAAOFJJnAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAADFBMVEUBAgMAAACA"
    "gID///9fpk1YAAAAAXRSTlMAQObYZgAAAAFiS0dEAxEMTPIAAAAHdElNRQfhCBIOEg85efYa"
    "AAAARUlEQVQY02NggAMHDEYDlBZLRRdhcIMxPBOgDMYwKIMpFCYUCjNpKky1CExEQQAmxMKA"
    "AdJSYNZxwJzEgs6AS8EV0wQAAFHyCNebrGE6AAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDE3LTA4"
    "LTE4VDE0OjE3OjM2KzAyOjAw2wadDQAAACV0RVh0ZGF0ZTptb2RpZnkAMjAxNy0wOC0xOFQx"
    "NDoxODoxNSswMjowMCidY9wAAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['DrawRectangle'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAACXBIWXMAAAsTAAALEwEAmpwY"
    "AAAA0UlEQVRYhe2WWwrEIAxF752NtUurO0tWduejONRaBwaMMmDAj6o0J8f0QUmYGa+p2RfA"
    "LwDuLncvrsMAGjcnALl7XmcYgJm19hNAt+SPAO6u4zgqC9u24ZK827NbAZgZJPFu4aa9H4Sk"
    "zzAznVMCAJmZrusRozCQqyepJwsh0ap+lIWPgVz9DS7cAvPHKGuvNjTme8Xje4DksE9kcQQp"
    "pdwABICUkoYdAXBWfk2473vVF2EA37QP6YFWkmgD//M/EBVFE86I6QYWwAJYAAtgOsAbi8Eu"
    "VUcv3/wAAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['DrawStart'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgP/AAAA"
    "AACAgID////e+dEAAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAASUlEQVQoz9XOsQ3AMAwDQYKeIBsI9AbZf7ikzQuIGzdWeSAekrbc1WACHMgIJakv"
    "3GGksBhy+6UI74RlMdQXCLus/IJsr6LH3QPwBQOyjzdP8gAAACV0RVh0ZGF0ZTpjcmVhdGUA"
    "MjAxNy0wOC0xOFQxNDoxNzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5ADIwMTct"
    "MDgtMThUMTQ6MTg6MTUrMDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditAnchor'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgPAwMAA"
    "AACAgID///9hUlqkAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAAVUlEQVQoz8WPuw2AQAxDTVgghAUiT3Dc/sNBy6OgQcLlU/yJ9Ik2gjoAVoMsJpkk"
    "ZTOkGRLoGcJJiifX3AGww6TZepUbyfGo6nbeLNnYFyqG8Mk/dQJ27gU0/VtoiAAAACV0RVh0"
    "ZGF0ZTpjcmVhdGUAMjAxNy0wOC0xOFQxNDoxNzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6"
    "bW9kaWZ5ADIwMTctMDgtMThUMTQ6MTg6MTUrMDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditComponent'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgP/AAAA"
    "AACAgID////e+dEAAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAAOklEQVQoz2NgoAoQxBBQRhNgUkITYVZCF1FSUkAVMFJCN0QBTQUzAxOGWxTQBTCV"
    "gExCE1CiuoJhAQAeOwLkdi2jxwAAACV0RVh0ZGF0ZTpjcmVhdGUAMjAxNy0wOC0xOFQxNDox"
    "NzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5ADIwMTctMDgtMThUMTQ6MTg6MTUr"
    "MDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditContour'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAElBMVEUBAgP/AAAA"
    "AACAgIDAwMD////dqDPgAAAAAXRSTlMAQObYZgAAAAFiS0dEBfhv6ccAAAAHdElNRQfhCBIO"
    "Eg85efYaAAAATUlEQVQoz2NgoAoQxBBQRhNgUkITYVZCF1FSUkAVMFJCN0QBTQUzAxOGWxQY"
    "nFFVMTEpGaCpUXJAV8KEbgghBU5oTsF0rbIDGSE5yAAAJSwE1hxasGwAAAAldEVYdGRhdGU6"
    "Y3JlYXRlADIwMTctMDgtMThUMTQ6MTc6MzYrMDI6MDDbBp0NAAAAJXRFWHRkYXRlOm1vZGlm"
    "eQAyMDE3LTA4LTE4VDE0OjE4OjE1KzAyOjAwKJ1j3AAAAABJRU5ErkJggg==").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditContourDirection'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAACXBIWXMAAAsTAAALEwEAmpwY"
    "AAABO0lEQVRYhe2WzU6FMBBGz8BNNMaFC13pq0Kn8KouXLhwZVwI44JrUktvpFx+NjSZkBSY"
    "7/AxnVbMjD1Hsav6AQCcRjMiBlAUpcxJ2HffWc9fdMD6zlxdrV+hZvYnpCgBLAxVNSlKpkSc"
    "779IAqiqxRBTQRYBSLkAJMVjqMUAYhcufXF8bzGA0IWU0OoOpJKnIIY6zhOdDDDFclW9CkDi"
    "zagox71ptHT7zuTcqKzvzMxmNS2Y2Yol6JKqOld7yDXHgXCs7oCrK/tty6mrqiIi5r03Oe8j"
    "OWN3Byarubp6ALrgHVPV581qwNXVCbj3TfsRzm/mAGCxOAwrIoTOPQ/kANwAn66uXoB34BEQ"
    "ht8iQOeb9i1LPRPAXF09Abe+ab+A11yxdNbMVhy14btrzwOjItx67H4qPgAOgAPgB+BLPQxY"
    "v+AIAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditGuideline'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgP/AAAA"
    "AACAgID////e+dEAAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAAQUlEQVQoz2NgoAoQxBBQRhNgUkITYVZCF1FSUkAVMFJCN0QBTQUzAxOGWxQYDAxQ"
    "dTGga0NXwYCuwggIKFUxLAAAxrMFaIj+lE0AAAAldEVYdGRhdGU6Y3JlYXRlADIwMTctMDgt"
    "MThUMTQ6MTc6MzYrMDI6MDDbBp0NAAAAJXRFWHRkYXRlOm1vZGlmeQAyMDE3LTA4LTE4VDE0"
    "OjE4OjE1KzAyOjAwKJ1j3AAAAABJRU5ErkJggg==").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditHints'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgAgMAAAAOFJJnAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAACVBMVEUBAgMAAAD/"
    "///na8ZxAAAAAXRSTlMAQObYZgAAAAFiS0dEAmYLfGQAAAAJcEhZcwAACxMAAAsTAQCanBgA"
    "AAAHdElNRQflCgENGjd4GZPSAAAAMElEQVQY02NgYGGAAlEYI9QBQjOGBkBF4GpgjNBQmCLc"
    "AKEGQzvCZLgxouhqBgcAAFpiBTfyCWVOAAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDE3LTA5LTE2"
    "VDE0OjU5OjE3KzAyOjAwZiSkBgAAACV0RVh0ZGF0ZTptb2RpZnkAMjAyMS0xMC0wMVQxMzoy"
    "Njo1NSswMjowMGSAvS8AAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 6)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 8)
cursors['EditHLine'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgPAwMAA"
    "AACAgID///9hUlqkAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAATElEQVQoz9WOsRHAIAwDBVkg4AU4TQDsPxzUT0GTJur8J8mSPlEhqAPgMUgyySSp"
    "NksaSzL+dMHyipY9twMEQppNdwXuxNSxV9a/tAAClgR90CbKlgAAACV0RVh0ZGF0ZTpjcmVh"
    "dGUAMjAxNy0wOC0xOFQxNDoxNzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5ADIw"
    "MTctMDgtMThUMTQ6MTg6MTUrMDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditPoint'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgP/AAAA"
    "AACAgID////e+dEAAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAAQUlEQVQoz92MsQ0AIAzDovQCPqjCB/x/HKxNBxYmPFpOgCeMJqYJykzIjZRVLPlJ"
    "WhHgobr0EWGFJ0Qr2gcvxadskjwCTqXjMFoAAAAldEVYdGRhdGU6Y3JlYXRlADIwMTctMDgt"
    "MThUMTQ6MTc6MzYrMDI6MDDbBp0NAAAAJXRFWHRkYXRlOm1vZGlmeQAyMDE3LTA4LTE4VDE0"
    "OjE4OjE1KzAyOjAwKJ1j3AAAAABJRU5ErkJggg==").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditRubberBand'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgPAwMAA"
    "AACAgID///9hUlqkAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAATklEQVQoz2NgoAoQRBcQMkQTYFZCE2FUQhcxQhcRUlJCN0QB3RAmNHsMGNCUCDCg"
    "KwE61wBNQBhNE4ORAoYmBSDAHwoKDAyEVBA0Y5ABAGheBPCRDxl5AAAAJXRFWHRkYXRlOmNy"
    "ZWF0ZQAyMDE3LTA4LTE4VDE0OjE3OjM2KzAyOjAw2wadDQAAACV0RVh0ZGF0ZTptb2RpZnkA"
    "MjAxNy0wOC0xOFQxNDoxODoxNSswMjowMCidY9wAAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditSelection'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAElBMVEUBAgP/AAAA"
    "AACAgIDAwMD////dqDPgAAAAAXRSTlMAQObYZgAAAAFiS0dEBfhv6ccAAAAHdElNRQfhCBIO"
    "Eg85efYaAAAATElEQVQoz2NgoAoQxBBQRhNgUkITYVZCF1FSUkAVMFJCN0QBTQUzAxOGWxRA"
    "6lB0QTGKGlSDmJjQVTijmcFigOZYJjTHYTiEsMAQAACxbgNpADg8sAAAACV0RVh0ZGF0ZTpj"
    "cmVhdGUAMjAxNy0wOC0xOFQxNDoxNzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5"
    "ADIwMTctMDgtMThUMTQ6MTg6MTUrMDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EditStartpoint'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAAAmJLR0QA/4ePzL8A"
    "AAAJcEhZcwAACxMAAAsTAQCanBgAAAAHdElNRQfhCQ8QMCMluPRRAAAAVUlEQVRIx+2QOw7A"
    "MAhD7Yj7X9kdUJM0G87SAUuwvceHEI6IKCROgKrgwEiIens1w538EYirLEEe4G0xT3CmA0As"
    "2FOE/77MHb3/oAUtaEELWvADwQNpWBRDo3v/gwAAACV0RVh0ZGF0ZTpjcmVhdGUAMjAxNy0w"
    "OS0xNVQxNjo0ODoyNiswMjowMCkEHRYAAAAldEVYdGRhdGU6bW9kaWZ5ADIwMTctMDktMTVU"
    "MTY6NDg6MzUrMDI6MDClG7+pAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['Erase'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAAFVBMVEX///8AAAD/"
    "AAABAgMAAAD/AAD////tUcaIAAAABHRSTlMAAAAAs5NmmgAAAAFiS0dEAIgFHUgAAAAJcEhZ"
    "cwAACxMAAAsTAQCanBgAAAAHdElNRQfhCQ8REgAhr6k0AAAAT0lEQVQoz2NwAQIHBiTgAMUI"
    "ARcWB1QBV1c0AREVBgIqXFQcUQUEGQRRBDDcgR0Ym7gYG6AKmKALoKswdkYVcDF2NnHAK2Ds"
    "7OyM39CRAgB3gRH88KbVTQAAACV0RVh0ZGF0ZTpjcmVhdGUAMjAxNy0wOS0xNVQxNzoxNzo1"
    "MyswMjowMCwzFeIAAAAldEVYdGRhdGU6bW9kaWZ5ADIwMTctMDktMTVUMTc6MTg6MDArMDI6"
    "MDDVbeIqAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EraseAnchor'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAAGFBMVEX///8AAAD/"
    "AAABAgMAAAD/AADAwMD////vANanAAAABHRSTlMAAAAAs5NmmgAAAAFiS0dEAIgFHUgAAAAJ"
    "cEhZcwAACxMAAAsTAQCanBgAAAAHdElNRQfhCQ8RGhj6GrtqAAAAVElEQVQoz2NwAQIHBiTg"
    "AMUIARcWB1QBV1c0AREVBQIqXFQcUQUElQRRBDDcQSRgcTF2NkYWYHZGFzAxMXFD0eJmYmyC"
    "osLY2cwZTcDEhQzHDD8AACN2Dw/moPVYAAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDE3LTA5LTE1"
    "VDE3OjI2OjIyKzAyOjAw9qEacAAAACV0RVh0ZGF0ZTptb2RpZnkAMjAxNy0wOS0xNVQxNzoy"
    "NjoyNCswMjowMOQsl/YAAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EraseComponent'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAACXBIWXMAAAsTAAALEwEAmpwY"
    "AAABaElEQVRYhe1Vu3KDMBDck2jieCYp0iW/Ckj2b+YDUqRIlSaGdQF4ZI2whHikYWc0QnCj"
    "XfbuJAFAREBSYjG5KGIEIhIVOAfKJRrI3Oe1odwFe/JNmAMCRDoR3bxi3scEkAAE4B9uadAA"
    "oHRxN6y1dNdz0P8sZch570BXC0oL24aitLTN5UY2vBs2aZtLtoC7FACQzgc8D6QhGGOyCX30"
    "aY8EBRxwa0TpAu73KY4UoWJz8xrabFEHGGg6v7BcEUoXyzqQEuQLGnMgpxiTHPAx5kAOVOhl"
    "XZVk23BsNsZARGitnX1k/7sDD5nqqnwF0DhxNMa8b9YFdVUWAI72dP5xv/sOzEGsC+iTA93B"
    "9KhVpyDoQAg5PZ6C7KtsKQeyBIS6JHZ8LybAWnsA8FtX5QeAbwBv6C61xpm/UvebXAN1VT4B"
    "OAJ4safzZyh2UjpIJg1RenQYYw7uOnVPkukOrIXgXbAL2AXsArbEFRtnHQSnzLlLAAAAAElF"
    "TkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['EraseGuideline'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAAG1BMVEX///8AAAD/"
    "AAABAgMAAAD/AADAwMCAgID///9GAK5TAAAABHRSTlMAAAAAs5NmmgAAAAFiS0dEAIgFHUgA"
    "AAAJcEhZcwAACxMAAAsTAQCanBgAAAAHdElNRQfhCQ8REzAebajZAAAAQklEQVQoz2NwAQIH"
    "BiTgAMUIARcWB1QBV1c0AREVBQIqXFQcUQUElQRRBDDcQTYwK0YTSHdBEyhBFzBxporNwxAA"
    "AIh1DAXD/Gj4AAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDE3LTA5LTE1VDE3OjE5OjQ2KzAyOjAw"
    "rGgKaAAAACV0RVh0ZGF0ZTptb2RpZnkAMjAxNy0wOS0xNVQxNzoxOTo0OCswMjowMI0KyYkA"
    "AAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['ErasePoint'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAAFVBMVEX///8AAACB"
    "gYH////CwsLBwcH/AADvh1QtAAAAAXRSTlMAQObYZgAAAAFiS0dEAIgFHUgAAAAJcEhZcwAA"
    "CxMAAAsTAQCanBgAAAAHdElNRQflCgEOJSyo5M5bAAAAQklEQVQoz2NgwA4YFdAEhAzQFCij"
    "CQgZO6ApMBVAERAWDkRVoGiIqkAoGJXPqGqMpsDZEM0EYfwK2AQFURWMAhIBAO1pBOc8FykE"
    "AAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDIxLTEwLTAxVDE0OjM3OjM5KzAyOjAwRy3UxgAAACV0"
    "RVh0ZGF0ZTptb2RpZnkAMjAyMS0xMC0wMVQxNDozNzo0NCswMjowMF1iBKMAAAAASUVORK5C"
    "YII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 1)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 13)
cursors['Knife'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgAgMAAAAOFJJnAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAADFBMVEUBAgP//wAA"
    "AAD///9yrVWEAAAAAXRSTlMAQObYZgAAAAFiS0dEAxEMTPIAAAAHdElNRQfhCBIOEg85efYa"
    "AAAAM0lEQVQY02NggAHGCVAGRweUIaQIZWh0oYvA1TBOaICymBbATFoBY2jBGFzEKOJgGIQA"
    "AEK4B1WaBPU9AAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDE3LTA4LTE4VDE0OjE3OjM2KzAyOjAw"
    "2wadDQAAACV0RVh0ZGF0ZTptb2RpZnkAMjAxNy0wOC0xOFQxNDoxODoxNSswMjowMCidY9wA"
    "AAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['MagicWand'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgMAAAC3"
    "t7dwcHD///8mxPxCAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAAMUlEQVQoz2NgYBBgQAWKggoofCYDASMUAUYGAUYUAUFBRkEBvCowzMCwBdMdo2AQ"
    "AADZSwIVWt8QKQAAACV0RVh0ZGF0ZTpjcmVhdGUAMjAxNy0wOC0xOFQxNDoxNzozNiswMjow"
    "MNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5ADIwMTctMDgtMThUMTQ6MTg6MTUrMDI6MDAonWPc"
    "AAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['MarkCenter'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JQAAgIMAAPn/AACA6QAAdTAAAOpgAAA6mAAAF2+SX8VGAAAAElBMVEX/////AAAA"
    "AACAgIDAwMD///9LuXgMAAAAAXRSTlMAQObYZgAAAAFiS0dEAIgFHUgAAAAJcEhZcwAACxMA"
    "AAsTAQCanBgAAAAHdElNRQfhCRISMx+ADYA5AAAAQklEQVQoz2NgoA1gFFJGEzBAU8HEhC7A"
    "oIAqwMzABBZGBkAlCmjKmBzQTFJRQDWJyYGFicoKGBTQHYvhHXQFQxMAAILDA6FStFwRAAAA"
    "JXRFWHRkYXRlOmNyZWF0ZQAyMDE3LTA5LTE4VDE4OjUxOjI5KzAyOjAwEXvXHAAAACV0RVh0"
    "ZGF0ZTptb2RpZnkAMjAxNy0wOS0xOFQxODo1MTozMSswMjowMJ9jIVkAAAAASUVORK5CYII=").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['Meter'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgPAwMAA"
    "AACAgID///9hUlqkAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAARklEQVQoz83OyRUAEBRD0aABQwNOKkD/xbF+thayvCd/kJ6kEOoEJEOCKYtSbS7p"
    "XBJxZwiVLFbOuwPQMKTVdScS/bzwWzYCsAR8GJqo7QAAACV0RVh0ZGF0ZTpjcmVhdGUAMjAx"
    "Ny0wOC0xOFQxNDoxNzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5ADIwMTctMDgt"
    "MThUMTQ6MTg6MTUrMDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['PointContour'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgOAgIAA"
    "AADAwMD////VfPkDAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAATElEQVQoz2NgoA5gVFJE4TMrMjAqIwswQTEcCAK5jAJIAgogeQVkASYlJVQBdBUY"
    "ZmDYAnKHErIKkEuZUN0KBKhKQIYQoYRKQTeoAQB44wMu4nAJLQAAACV0RVh0ZGF0ZTpjcmVh"
    "dGUAMjAxNy0wOC0xOFQxNDoxNzozNiswMjowMNsGnQ0AAAAldEVYdGRhdGU6bW9kaWZ5ADIw"
    "MTctMDgtMThUMTQ6MTg6MTUrMDI6MDAonWPcAAAAAElFTkSuQmCC").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['ZoomIn'] = wx.Cursor(image)

image = PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgBAMAAACBVGfHAAAABGdBTUEAALGPC/xhBQAAACBj"
    "SFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAD1BMVEUBAgOAgIAA"
    "AADAwMD////VfPkDAAAAAXRSTlMAQObYZgAAAAFiS0dEBI9o2VEAAAAHdElNRQfhCBIOEg85"
    "efYaAAAASklEQVQoz2NgoA5gVFJE4TMrMjAqIwswQTEcCIJ0CSAJKMAJmACjkhKqALoKDDMw"
    "bAG5QwlZBcilTKhuBQJUJSBDiFBCpaAb1AAAa1ADJb7/oi4AAAAldEVYdGRhdGU6Y3JlYXRl"
    "ADIwMTctMDgtMThUMTQ6MTc6MzYrMDI6MDDbBp0NAAAAJXRFWHRkYXRlOm1vZGlmeQAyMDE3"
    "LTA4LTE4VDE0OjE4OjE1KzAyOjAwKJ1j3AAAAABJRU5ErkJggg==").GetImage()
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_X, 5)
image.SetOption(wx.IMAGE_OPTION_CUR_HOTSPOT_Y, 5)
cursors['ZoomOut'] = wx.Cursor(image)


del PyEmbeddedImage
del image



"""
markColor
===============================================================================
"""
import logging
import wx
import wx.aui as aui
from wx.lib.colourdb import updateColourDB, getColourList

from wbDefcon.objects.color import Color, mark

from ..view.font import UfoFontView
from ..view.font.groups import GroupsPage
from ..view.fontinfo import UfoFontInfoView
from ..view.glyph import UfoGlyphView

log = logging.getLogger(__name__)

updateColourDB()


class ColorMixin:
    @property
    def app(self):
        return wx.GetApp()

    @property
    def font(self):
        return self.app.documentManager.currentView.font

    @staticmethod
    def bitmap(color):
        return wx.Bitmap.FromRGBA(16, 16, *wx.Colour(color))

    @property
    def glyphs(self):
        view = self.app.documentManager.currentView
        if isinstance(view, UfoFontView):
            return self.font.selectedGlyphs
        if isinstance(view, UfoFontInfoView):
            return [self.font[g] for g in self.font.groups[view.frame.CurrentPage.currentGroupName]]
        if isinstance(view, UfoGlyphView):
            return [view.glyph]

    def markSelectedGlyphs(self, color):
        log.debug("color: %r", color)
        font = self.font
        font.holdNotifications()
        if color in ("WHITE", "No Color"):
            c = None
        else:
            c = Color(wx.Colour(color))
        for glyph in self.glyphs:
            glyph.markColor = c
        font.releaseHeldNotifications()


class ColorMenu(wx.Menu, ColorMixin):
    def __init__(self):
        super().__init__()
        for color in getColourList():
            if (
                " " not in color
                and color[-1] not in "01234567899"
                and color not in ("WHITE", "BLACK")
            ):
                item = wx.MenuItem(self, wx.ID_ANY, color)
                item.SetBitmap(self.bitmap(color))
                self.Append(item)
                self.Bind(wx.EVT_MENU, self.on_menu, id=item.GetId())

    def on_menu(self, event):
        self.markSelectedGlyphs(self.GetLabelText(event.Id))


class GlyphMarkToolbar(aui.AuiToolBar, ColorMixin):
    def __init__(self, parent):
        id = wx.ID_ANY
        pos = wx.DefaultPosition
        size = wx.DefaultSize
        style = (
            wx.aui.AUI_TB_HORZ_LAYOUT | wx.aui.AUI_TB_PLAIN_BACKGROUND | wx.NO_BORDER
        )
        aui.AuiToolBar.__init__(self, parent, id, pos, size, style)
        self.SetToolBitmapSize(wx.Size(16, 16))
        tool = self.AddTool(
            wx.ID_ANY, "No Color", self.bitmap("WHITE"), "No Color", wx.ITEM_NORMAL
        )
        self.Bind(wx.EVT_TOOL, self.on_Tool, tool)
        self.Bind(wx.EVT_UPDATE_UI, self.on_update_Tool, tool)
        for color in ("RED", "ORANGE", "YELLOW", "GREEN", "CYAN", "BLUE", "MAGENTA"):
            tool = self.AddTool(
                wx.ID_ANY, color, self.bitmap(color), color, wx.ITEM_NORMAL
            )
            self.Bind(wx.EVT_TOOL, self.on_Tool, tool)
            self.Bind(wx.EVT_UPDATE_UI, self.on_update_Tool, tool)
        self.AddSeparator()
        # add ColorMenu
        tool = self.AddTool(
            wx.ID_ANY,
            "Select Color",
            self.bitmap("GREY"),
            "Select Color",
            wx.ITEM_NORMAL,
        )
        tool.SetHasDropDown(True)
        self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.on_ColorMenu, tool)
        self.Bind(wx.EVT_UPDATE_UI, self.on_update_Tool, tool)
        self.menu = ColorMenu()

    def on_Tool(self, event):
        self.markSelectedGlyphs(self.GetToolLabel(event.Id))

    def on_ColorMenu(self, event):
        eventID = event.Id
        self.SetToolSticky(eventID, True)
        rect = self.GetToolRect(eventID)
        pt = self.ClientToScreen(rect.GetBottomLeft())
        pt = self.ScreenToClient(pt)
        self.PopupMenu(self.menu, pt)
        self.SetToolSticky(eventID, False)

    def on_update_Tool(self, event):
        view = self.app.documentManager.currentView
        if view:
            if isinstance(view, UfoFontView):
                event.Enable(True)
            elif isinstance(view, UfoFontInfoView):
                if isinstance(view.frame.CurrentPage, GroupsPage) and view.frame.CurrentPage.currentGroupName:
                    event.Enable(True)
                else:
                    event.Enable(False)
            elif isinstance(view, UfoGlyphView):
                event.Enable(True)
        else:
            event.Enable(False)



"""
applyGlyphConstructionDialog
===============================================================================
"""
import wx

from wbDefcon import Color

from .applyGlyphConstructionDialogUI import ApplyGlyphConstructionDialogUI


class ApplyGlyphConstructionDialog(ApplyGlyphConstructionDialogUI):
    def __init__(self):
        super().__init__(wx.GetApp().TopWindow)

    def update_SaveOutline(self, event):
        event.Enable(self.checkBox_ReplaceOutline.Value)

    def update_Mark(self, event):
        event.Enable(self.checkBox_Mark.Value)

    @property
    def constructionPath(self):
        return self.filePicker_ConstructionPath.GetPath()

    @property
    def newGlyphs(self):
        return self.checkBox_NewGlyphs.Value

    @property
    def replaceOutline(self):
        return self.checkBox_ReplaceOutline.Value

    @property
    def saveOutline(self):
        return self.checkBox_ReplaceOutline.Value and self.checkBox_SaveOutline.Value

    @property
    def replaceComposite(self):
        return self.checkBox_ReplaceComposite.Value

    @property
    def markColor(self):
        if self.checkBox_Mark.Value:
            return Color.from_wx(self.colourPicker_Mark.Colour)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Oct 26 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class NewFontDialogUI
###########################################################################

class NewFontDialogUI ( wx.Dialog ):

	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"New Font", pos = wx.DefaultPosition, size = wx.Size( 300,-1 ), style = wx.CAPTION|wx.RESIZE_BORDER )

		self.SetSizeHints( wx.Size( 200,-1 ), wx.DefaultSize )

		sizer_main = wx.BoxSizer( wx.VERTICAL )

		sizer_form = wx.FlexGridSizer( 0, 2, 0, 0 )
		sizer_form.AddGrowableCol( 1 )
		sizer_form.SetFlexibleDirection( wx.HORIZONTAL )
		sizer_form.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.label_family = wx.StaticText( self, wx.ID_ANY, u"Family Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_family.Wrap( -1 )

		sizer_form.Add( self.label_family, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.textCtrl_family = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		sizer_form.Add( self.textCtrl_family, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

		self.label_style = wx.StaticText( self, wx.ID_ANY, u"Style Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_style.Wrap( -1 )

		sizer_form.Add( self.label_style, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.textCtrl_style = wx.TextCtrl( self, wx.ID_ANY, u"Regular", wx.DefaultPosition, wx.DefaultSize, 0 )
		sizer_form.Add( self.textCtrl_style, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

		self.label_upm = wx.StaticText( self, wx.ID_ANY, u"UPM-Size", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label_upm.Wrap( -1 )

		sizer_form.Add( self.label_upm, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.spinCtrl_upm = wx.SpinCtrl( self, wx.ID_ANY, u"1000", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 10000, 1000 )
		sizer_form.Add( self.spinCtrl_upm, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		sizer_main.Add( sizer_form, 1, wx.EXPAND, 5 )

		sizer_dialog = wx.StdDialogButtonSizer()
		self.sizer_dialogOK = wx.Button( self, wx.ID_OK )
		sizer_dialog.AddButton( self.sizer_dialogOK )
		self.sizer_dialogCancel = wx.Button( self, wx.ID_CANCEL )
		sizer_dialog.AddButton( self.sizer_dialogCancel )
		sizer_dialog.Realize();

		sizer_main.Add( sizer_dialog, 0, wx.ALL|wx.EXPAND, 5 )


		self.SetSizer( sizer_main )
		self.Layout()

		self.Centre( wx.BOTH )

	def __del__( self ):
		pass





"""
magicwand
===============================================================================
"""

import wx

from ..cursor import cursors
from .base import GlyphTool


class MagicWand(GlyphTool):
    cursor = cursors["MagicWand"]

    def __init__(self, parent):
        GlyphTool.__init__(self, parent)
        self.addToSelection = False
        # Mouse Events
        self.Bind(wx.EVT_MOTION, self.on_MOTION)
        # Keyboard Events
        self.Bind(wx.EVT_KEY_DOWN, self.on_KEY_DOWN)
        self.Bind(wx.EVT_KEY_UP, self.on_KEY_UP)

    def reset(self):
        super(MagicWand, self).reset()
        self.addToSelection = False

    def on_LEFT_DOWN(self, event):
        GlyphTool.on_LEFT_DOWN(self, event)
        x = self.canvas.screenToCanvasX(event.GetX())
        y = self.canvas.screenToCanvasY(event.GetY())
        contourDistance = {}
        for contour in self.glyph:
            minDistance = 100000
            for point in contour:
                minDistance = min(minDistance, point.distance((x, y)))
            contourDistance[minDistance] = contour
        if not self.addToSelection:
            self.unselectAll()
        contourDistance[min(contourDistance.keys())].selected = True
        self.canvas.Refresh()
        event.Skip()

    def on_MOTION(self, event):
        if not event.ControlDown():
            self.canvas.unselectTool()
        GlyphTool.on_MOTION(self, event)

    def on_KEY_DOWN(self, event):
        unicodeKey = event.GetUnicodeKey()
        if unicodeKey == wx.WXK_NONE:
            key = event.KeyCode
        else:
            key = chr(unicodeKey)
        if key == " ":  # Space pressed
            self.canvas.selectTool("ToolZoom")
        elif key == wx.WXK_SHIFT:
            self.addToSelection = True
        event.Skip()

    def on_KEY_UP(self, event):
        unicodeKey = event.GetUnicodeKey()
        if unicodeKey == wx.WXK_NONE:
            key = event.KeyCode
        else:
            key = chr(unicodeKey)
        if key == wx.WXK_SHIFT:
            self.addToSelection = False
        event.Skip()



"""
tooldraw
===============================================================================

Implementation of the draw tool for the glyph editor
"""
import logging

from wbDefcon import Contour, Point

from ..cursor import cursors
from .base import IDLE, WORKING, GlyphTool

log = logging.getLogger(__name__)


class ToolDraw(GlyphTool):
    """
    Tool to draw contours
    """

    cursor = cursors["DrawStart"]

    def __init__(self, parent):
        super().__init__(parent)
        self.currentContour = None
        self.startPoint = None

    def reset(self):
        print("ToolDraw.reset")
        super().reset()
        self.currentContour = None
        self.startPoint = None

    def setCursor(self):
        # if isinstance(self.subject, Point):
        if self.subject is None:
            if self.startPoint is None:
                self.canvas.SetCursor(cursors["DrawStart"])
            else:
                self.canvas.SetCursor(cursors["Draw"])

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_LEFT_DOWN(self, event):
        super().on_LEFT_DOWN(event)
        glyph = self.glyph
        if glyph is not None:
            stack = self.activeStack
            if not stack["OnCurvePoints"].locked:
                glyph.undoManager.saveState()
                if self.currentContour is None:
                    self.currentContour = Contour()
                    glyph.appendContour(self.currentContour)
                x1 = self.canvas.screenToCanvasX(self.x)
                y1 = self.canvas.screenToCanvasY(self.y)
                newPoint = Point((x1, y1), "line")
                if self.startPoint is None:
                    newPoint.segmentType = "move"
                    self.startPoint = newPoint
                self.currentContour.appendPoint(newPoint)
                self.state = WORKING
        event.Skip()

    def on_LEFT_UP(self, event):
        if self.canvas.HasCapture():
            self.canvas.ReleaseMouse()
        # self.reset()
        glyph = self.glyph
        if glyph is not None:
            try:
                glyph.enableNotifications()
            except KeyError:
                pass
        self.canvas.Refresh()
        event.Skip()

    def on_MOTION(self, event):
        super().on_MOTION(event)
        glyph = self.glyph
        if glyph is not None:
            canvas_x = self.canvas.screenToCanvasX(self.x)
            canvas_y = self.canvas.screenToCanvasY(self.y)
            if event.Dragging() and self.state == WORKING:
                dx = self.canvas.screenToCanvasXrel(self.x - self.xPrev)
                dy = self.canvas.screenToCanvasYrel(self.y - self.yPrev)
            else:
                if self.state == IDLE:
                    self.hitTest()
                    self.setCursor()
        event.Skip()



"""All tools available on canvas"""
from .addellipse import AddEllipse
from .addrectangle import AddRectangle
from .magicwand import MagicWand
from .tooldraw import ToolDraw
from .tooledit import ToolEdit
from .toolerase import ToolErase
from .toolknife import ToolKnife
from .toolmeter import ToolMeter
from .toolpan import ToolPan
from .toolrotate import ToolRotate
from .toolscale import ToolScale
from .toolstartpoint import ToolStartpoint
from .toolreverse import ToolReverse
from .toolzoom import ToolZoom



"""
toolstartpoint
===============================================================================

Implementation of the set start point tool for the glyph editor
"""
import logging

from wbDefcon import Point

from ..cursor import cursors
from .base import GlyphTool

log = logging.getLogger(__name__)


class ToolStartpoint(GlyphTool):
    """
    Tool to set start point of contours
    """
    cursor = cursors["EditStartpoint"]

    def hitTest(self):
        # check for pointing Stuff under cursor
        canvas_x = self.canvas.screenToCanvasX(self.x)
        canvas_y = self.canvas.screenToCanvasY(self.y)
        pointed = None
        self.subject = None
        # perform hit test on On Curve Points
        plain = self.activeStack.get("OnCurvePoints")
        if plain.visible and not plain.locked:
            pointed = plain.hitTest(canvas_x, canvas_y)
            if pointed is not None and pointed != self.subject:
                self.subject = pointed

    def setCursor(self):
        if isinstance(self.subject, Point):
            self.canvas.SetCursor(cursors["EditPoint"])
        else:
            self.canvas.SetCursor(self.cursor)

    # -------------------------------------------------------------------------
    # Event handler methods
    # -------------------------------------------------------------------------

    def on_MOTION(self, event):
        super().on_MOTION(event)
        glyph = self.glyph
        if glyph is not None:
            self.hitTest()
            self.setCursor()
        event.Skip()

    def on_LEFT_DOWN(self, event):
        if self.subject is not None:
            glyph = self.glyph
            for contour in glyph:
                if self.subject in contour:
                    glyph.undoManager.saveState()
                    contour.setStartPoint(contour.index(self.subject))



"""
anchorDialog
===============================================================================
"""
import wx

from wbDefcon import Color

from .anchorDialogUI import AnchorDialogUI


class AnchorDialog(AnchorDialogUI):
    def __init__(self, parent, anchorDict=None):
        super().__init__(parent)
        self.textCtrl_name.Hint = "None"
        self.dialogButtonsOK.SetDefault()
        if anchorDict is None:
            anchorDict = {}
        self.textCtrl_name.Value = anchorDict.get("name", "")
        self.spinCtrl_x.Value = round(anchorDict.get("x", 0))
        self.spinCtrl_y.Value = round(anchorDict.get("y", 0))
        color = anchorDict.get("color")
        if color:
            self.checkBox_color.Value = True
            self.colourPicker.SetColour(color.wx)
        else:
            self.checkBox_color.Value = False

    @property
    def anchorDict(self):
        result = dict(
            x=self.spinCtrl_x.Value,
            y=self.spinCtrl_y.Value,
            name=self.textCtrl_name.Value,
            color=None,
        )
        if self.colourPicker.IsEnabled():
            result["color"] = Color.from_wx(self.colourPicker.GetColour())
        return result

    def on_update_colourPicker(self, event):
        event.Enable(self.checkBox_color.Value)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoLineUI
###########################################################################

class FontInfoLineUI ( wx.ScrolledWindow ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.HSCROLL|wx.VSCROLL, name = wx.EmptyString ):
        wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetExtraStyle( wx.WS_EX_VALIDATE_RECURSIVELY )
        self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )

        sizerMain = wx.BoxSizer( wx.VERTICAL )

        boxPostScript = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"PostScript" ), wx.VERTICAL )

        sizerPostScript = wx.GridBagSizer( 0, 0 )
        sizerPostScript.SetFlexibleDirection( wx.HORIZONTAL )
        sizerPostScript.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_postscriptUnderlinePosition = wx.StaticText( self, wx.ID_ANY, u"Underline Position", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptUnderlinePosition.Wrap( -1 )

        self.lbl_postscriptUnderlinePosition.SetMinSize( wx.Size( 170,-1 ) )

        sizerPostScript.Add( self.lbl_postscriptUnderlinePosition, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptUnderlinePosition = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptUnderlinePosition" )
        sizerPostScript.Add( self.txtCtrl_postscriptUnderlinePosition, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_postscriptUnderlineThickness = wx.StaticText( self, wx.ID_ANY, u"Underline Thickness", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_postscriptUnderlineThickness.Wrap( -1 )

        sizerPostScript.Add( self.lbl_postscriptUnderlineThickness, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_postscriptUnderlineThickness = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptUnderlineThickness" )
        sizerPostScript.Add( self.txtCtrl_postscriptUnderlineThickness, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizerPostScript.AddGrowableCol( 1 )

        boxPostScript.Add( sizerPostScript, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxPostScript, 0, wx.ALL|wx.EXPAND, 5 )

        boxOS2 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"OpenType OS/2 Table" ), wx.VERTICAL )

        sizerOS2 = wx.GridBagSizer( 0, 0 )
        sizerOS2.SetFlexibleDirection( wx.HORIZONTAL )
        sizerOS2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_openTypeOS2StrikeoutPosition = wx.StaticText( self, wx.ID_ANY, u"Strikeout Position", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2StrikeoutPosition.Wrap( -1 )

        self.lbl_openTypeOS2StrikeoutPosition.SetMinSize( wx.Size( 170,-1 ) )

        sizerOS2.Add( self.lbl_openTypeOS2StrikeoutPosition, wx.GBPosition( 0, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2StrikeoutPosition = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2StrikeoutPosition" )
        sizerOS2.Add( self.txtCtrl_openTypeOS2StrikeoutPosition, wx.GBPosition( 0, 1 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_openTypeOS2StrikeoutSize = wx.StaticText( self, wx.ID_ANY, u"Strikeout Size", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_openTypeOS2StrikeoutSize.Wrap( -1 )

        sizerOS2.Add( self.lbl_openTypeOS2StrikeoutSize, wx.GBPosition( 1, 0 ), wx.GBSpan( 1, 1 ), wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.txtCtrl_openTypeOS2StrikeoutSize = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeOS2StrikeoutSize" )
        sizerOS2.Add( self.txtCtrl_openTypeOS2StrikeoutSize, wx.GBPosition( 1, 1 ), wx.GBSpan( 1, 1 ), wx.ALL, 5 )


        sizerOS2.AddGrowableCol( 1 )

        boxOS2.Add( sizerOS2, 1, wx.EXPAND, 5 )


        sizerMain.Add( boxOS2, 0, wx.ALL|wx.EXPAND, 5 )


        self.SetSizer( sizerMain )
        self.Layout()
        sizerMain.Fit( self )

    def __del__( self ):
        pass





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class GroupsStatusPanelUI
###########################################################################

class GroupsStatusPanelUI ( wx.Panel ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.BORDER_NONE|wx.TAB_TRAVERSAL, name = wx.EmptyString ):
        wx.Panel.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        sizer_outer = wx.BoxSizer( wx.VERTICAL )

        self.staticline = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        sizer_outer.Add( self.staticline, 0, wx.EXPAND, 0 )

        sizer = wx.BoxSizer( wx.HORIZONTAL )

        self.staticText = wx.StaticText( self, wx.ID_ANY, u"Show", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.staticText.Wrap( -1 )

        sizer.Add( self.staticText, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT, 5 )

        choiceGroupsChoices = [ u"All Groups", u"All Kerning Groups", u"Side 1 Kerning Groups", u"Side 2 Kerning Groups", u"All non Kerning Groups" ]
        self.choiceGroups = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choiceGroupsChoices, 0 )
        self.choiceGroups.SetSelection( 0 )
        sizer.Add( self.choiceGroups, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT, 5 )

        self.button_New = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_New.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_NEW, wx.ART_BUTTON ) )
        self.button_New.SetToolTip( u"Add new Group" )

        sizer.Add( self.button_New, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT, 5 )

        self.button_Delete = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_Delete.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_DELETE, wx.ART_BUTTON ) )
        self.button_Delete.SetToolTip( u"Delete current Group" )

        sizer.Add( self.button_Delete, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT, 5 )


        sizer.Add( ( 0, 0), 1, wx.EXPAND, 5 )

        self.button_macro = wx.Button( self, wx.ID_ANY, u"▼", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )

        self.button_macro.SetBitmap( wx.ArtProvider.GetBitmap( "PYTHON", wx.ART_BUTTON ) )
        sizer.Add( self.button_macro, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizer_outer.Add( sizer, 1, wx.EXPAND, 5 )


        self.SetSizer( sizer_outer )
        self.Layout()
        sizer_outer.Fit( self )

        # Connect Events
        self.choiceGroups.Bind( wx.EVT_CHOICE, self.on_choiceGroups )
        self.choiceGroups.Bind( wx.EVT_UPDATE_UI, self.update_choiceGroups )
        self.button_New.Bind( wx.EVT_BUTTON, self.on_button_New )
        self.button_Delete.Bind( wx.EVT_BUTTON, self.on_button_Delete )
        self.button_Delete.Bind( wx.EVT_UPDATE_UI, self.update_button_Delete )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def on_choiceGroups( self, event ):
        event.Skip()

    def update_choiceGroups( self, event ):
        event.Skip()

    def on_button_New( self, event ):
        event.Skip()

    def on_button_Delete( self, event ):
        event.Skip()

    def update_button_Delete( self, event ):
        event.Skip()





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.1-0-g8feb16b3)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class FontInfoIdentificationUI
###########################################################################

class FontInfoIdentificationUI ( wx.ScrolledWindow ):

	def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,438 ), style = wx.TAB_TRAVERSAL, name = wx.EmptyString ):
		wx.ScrolledWindow.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

		self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )

		sizer = wx.BoxSizer( wx.VERTICAL )

		boxMain = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Main" ), wx.VERTICAL )

		gridMain = wx.FlexGridSizer( 0, 2, 0, 0 )
		gridMain.AddGrowableCol( 1 )
		gridMain.SetFlexibleDirection( wx.BOTH )
		gridMain.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.lbl_versionMajor = wx.StaticText( boxMain.GetStaticBox(), wx.ID_ANY, u"Version Major", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_versionMajor.Wrap( -1 )

		self.lbl_versionMajor.SetMinSize( wx.Size( 170,-1 ) )

		gridMain.Add( self.lbl_versionMajor, 0, wx.ALL, 5 )

		self.txtCrtl_versionMajor = wx.TextCtrl( boxMain.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"versionMajor" )
		gridMain.Add( self.txtCrtl_versionMajor, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.lbl_versionMinor = wx.StaticText( boxMain.GetStaticBox(), wx.ID_ANY, u"Version Minor", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_versionMinor.Wrap( -1 )

		gridMain.Add( self.lbl_versionMinor, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.txtCrtl_versionMinor = wx.TextCtrl( boxMain.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"versionMinor" )
		gridMain.Add( self.txtCrtl_versionMinor, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.lbl_year = wx.StaticText( boxMain.GetStaticBox(), wx.ID_ANY, u"Year", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_year.Wrap( -1 )

		gridMain.Add( self.lbl_year, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.txtCtrl_year = wx.TextCtrl( boxMain.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"year" )
		gridMain.Add( self.txtCtrl_year, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		boxMain.Add( gridMain, 1, wx.EXPAND, 5 )


		sizer.Add( boxMain, 0, wx.ALL|wx.EXPAND, 5 )

		boxOTname = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"OpenType Name Table" ), wx.VERTICAL )

		gridOTname = wx.FlexGridSizer( 0, 2, 0, 0 )
		gridOTname.AddGrowableCol( 1 )
		gridOTname.AddGrowableRow( 2 )
		gridOTname.AddGrowableRow( 3 )
		gridOTname.SetFlexibleDirection( wx.BOTH )
		gridOTname.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.lbl_openTypeNameVersion = wx.StaticText( boxOTname.GetStaticBox(), wx.ID_ANY, u"Version String", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_openTypeNameVersion.Wrap( -1 )

		self.lbl_openTypeNameVersion.SetMinSize( wx.Size( 170,-1 ) )

		gridOTname.Add( self.lbl_openTypeNameVersion, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.txtCrtl_openTypeNameVersion = wx.TextCtrl( boxOTname.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNameVersion" )
		gridOTname.Add( self.txtCrtl_openTypeNameVersion, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

		self.lbl_openTypeNameUniqueID = wx.StaticText( boxOTname.GetStaticBox(), wx.ID_ANY, u"Unique ID", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_openTypeNameUniqueID.Wrap( -1 )

		gridOTname.Add( self.lbl_openTypeNameUniqueID, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.txtCrtl_openTypeNameUniqueID = wx.TextCtrl( boxOTname.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"openTypeNameUniqueID" )
		gridOTname.Add( self.txtCrtl_openTypeNameUniqueID, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )

		self.lbl_openTypeNameDescription = wx.StaticText( boxOTname.GetStaticBox(), wx.ID_ANY, u"Description", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_openTypeNameDescription.Wrap( -1 )

		gridOTname.Add( self.lbl_openTypeNameDescription, 0, wx.ALL, 5 )

		self.txtCtrl_openTypeNameDescription = wx.TextCtrl( boxOTname.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE, wx.DefaultValidator, u"openTypeNameDescription" )
		gridOTname.Add( self.txtCtrl_openTypeNameDescription, 1, wx.ALL|wx.EXPAND, 5 )

		self.lbl_openTypeNameSampleText = wx.StaticText( boxOTname.GetStaticBox(), wx.ID_ANY, u"Sample Text", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_openTypeNameSampleText.Wrap( -1 )

		gridOTname.Add( self.lbl_openTypeNameSampleText, 0, wx.ALL, 5 )

		self.txtCtrl_openTypeNameSampleText = wx.TextCtrl( boxOTname.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE, wx.DefaultValidator, u"openTypeNameSampleText" )
		gridOTname.Add( self.txtCtrl_openTypeNameSampleText, 1, wx.ALL|wx.EXPAND, 5 )


		boxOTname.Add( gridOTname, 1, wx.EXPAND, 5 )


		sizer.Add( boxOTname, 1, wx.ALL|wx.EXPAND, 5 )

		boxPostScript = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"PostScript" ), wx.VERTICAL )

		gridPostScript = wx.FlexGridSizer( 0, 2, 0, 0 )
		gridPostScript.AddGrowableCol( 1 )
		gridPostScript.SetFlexibleDirection( wx.BOTH )
		gridPostScript.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

		self.lbl_postscriptUniqueID = wx.StaticText( boxPostScript.GetStaticBox(), wx.ID_ANY, u"Unique ID", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.lbl_postscriptUniqueID.Wrap( -1 )

		self.lbl_postscriptUniqueID.SetMinSize( wx.Size( 170,-1 ) )

		gridPostScript.Add( self.lbl_postscriptUniqueID, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

		self.txtCtrl_postscriptUniqueID = wx.TextCtrl( boxPostScript.GetStaticBox(), wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0, wx.DefaultValidator, u"postscriptUniqueID" )
		gridPostScript.Add( self.txtCtrl_postscriptUniqueID, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )


		boxPostScript.Add( gridPostScript, 1, wx.EXPAND, 5 )


		sizer.Add( boxPostScript, 0, wx.ALL|wx.EXPAND, 5 )


		self.SetSizer( sizer )
		self.Layout()

	def __del__( self ):
		pass





"""
glyph
===============================================================================
"""
import logging
from math import atan2, pi

import wx
from fontTools.pens.transformPen import TransformPen
from wbBase.tools import get_wxBrush, get_wxFont, get_wxPen
from wbDefcon.pens import ContourHit, HitTestPen, graphicsPen, graphicsRoundingPen

from ....config import (
    NODESHAPE_CIRCLE,
    NODESHAPE_DIAMOND,
    NODESHAPE_SQUARE,
    NODESHAPE_STAR,
    NODESHAPE_TRIANGLE,
    cfgBrush,
    cfgPen,
    cfgShape,
    cfgSize,
)
from .base import DrawingPlane, DrawingPlaneStack, GuidelinePlaneMixin

log = logging.getLogger(__name__)

SelectionPen = wx.RED_PEN


class GlyphLevelPlain(DrawingPlane):
    """
    Base class for all glyph level drawing planes
    """

    visibleActiveDflt = True
    visibleInactiveDflt = False

    def __init__(self, parent, name):
        super().__init__(parent, name)
        self._visibleActive = self.visibleActiveDflt
        self._visibleInactive = self.visibleInactiveDflt

    @property
    def visible(self):
        active = self.parent.active
        return (active and self._visibleActive) or (
            self._visibleInactive and not active
        )

    @property
    def visibleActive(self):
        return self._visibleActive

    @visibleActive.setter
    def visibleActive(self, value):
        newValue = bool(value)
        if newValue != self._visibleActive:
            visible = self.visible
            self._visibleActive = newValue
            if self.visible != visible:
                self.canvas.Refresh()

    @property
    def visibleInactive(self):
        return self._visibleInactive

    @visibleInactive.setter
    def visibleInactive(self, value):
        newValue = bool(value)
        if newValue != self._visibleInactive:
            visible = self.visible
            self._visibleInactive = newValue
            if self.visible != visible:
                self.canvas.Refresh()

    @property
    def glyphName(self):
        return self.parent.parent.glyph.name

    @property
    def layer(self):
        return self.parent.parent.font.layers[self.parent.name]

    @property
    def glyph(self):
        g = self.parent.parent.glyph
        if g is not None:
            glyphName = g.name
            layer = self.parent.parent.font.layers[self.parent.name]
            if glyphName in layer:
                return layer[glyphName]


class LabelPlain(GlyphLevelPlain):
    """
    Base class to draw labels of points and anchors on Canvas.
    Not intended to be intantiated directly.
    """

    visibleActiveDflt = False
    visibleInactiveDflt = False
    lockedDflt = True
    LabelFont = get_wxFont(pointSize=7, faceName="Small Fonts")
    LabelPen = wx.TRANSPARENT_PEN
    LabelBrush = get_wxBrush(wx.Colour(230, 230, 240, 200))
    # Label types
    LT_POSITION = 1
    LT_NAME = 3
    LT_IDENTFIER = 3

    def __init__(self, parent, name):
        super().__init__(parent, name)
        self.labelType = self.LT_POSITION

    @property
    def points(self):
        "Needs to be implemented by subclasses"
        raise NotImplementedError

    def draw(self, gc):
        gc.SetBrush(self.LabelBrush)
        gc.SetPen(self.LabelPen)
        gc.SetFont(self.LabelFont, wx.BLACK)
        d = 3  # distace between point and label
        s = 2  # space around text
        for point in self.points:
            if self.labelType == self.LT_POSITION:
                text = f"{point.x:.0f} / {point.y:.0f}"
            elif self.labelType == self.LT_NAME:
                if not point.name:
                    continue
                text = point.name
            elif self.labelType == self.LT_IDENTFIER:
                if not point.identifier:
                    continue
                text = point.identifier
            w, h = gc.GetTextExtent(text)
            w = int(round(w)) + 2 * s
            h = int(round(h)) + 2 * s
            x, y = self.transform.transformPoint((point.x, point.y))
            x = round(x)
            y = round(y)
            gc.DrawRoundedRectangle(x + d, y + d, w, h, d)
            gc.DrawText(text, x + d + s, y + d + s - 1)


class ShapeDrawPlain(GlyphLevelPlain):
    """Base class for nodes and anchors"""

    def drawShape(self, gc, point, node):
        x, y = self.transform.transformPoint((point.x, point.y))
        x = round(x)
        y = round(y)
        if hasattr(point, "color") and point.color is not None:
            brush = get_wxBrush(point.color.wx)
        else:
            brush = node["brush"]
        if self.parent.active:
            if point.selected:
                gc.SetPen(SelectionPen)
            else:
                gc.SetPen(node["pen"])
            gc.SetBrush(brush)
        else:
            gc.SetBrush(self.getInactiveBrush(brush))
            gc.SetPen(self.getInactivePen(node["pen"]))
        shape = node["shape"]
        s = node["size"]
        if shape == NODESHAPE_CIRCLE:
            gc.DrawEllipse(x - s, y - s, 2 * s, 2 * s)
        elif shape == NODESHAPE_SQUARE:
            gc.DrawRectangle(x - s, y - s, 2 * s, 2 * s)
        elif shape == NODESHAPE_DIAMOND:
            gc.DrawLines([(x - s, y), (x, y - s), (x + s, y), (x, y + s), (x - s, y)])
        elif shape == NODESHAPE_TRIANGLE:
            gc.DrawLines([(x - s, y + s), (x, y - s), (x + s, y + s), (x - s, y + s)])
        elif shape == NODESHAPE_STAR:
            if s > 7:
                i = 2
            else:
                i = 1
            gc.DrawLines(
                [
                    (x - s, y),
                    (x - i, y - i),
                    (x, y - s),
                    (x + i, y - i),
                    (x + s, y),
                    (x + i, y + i),
                    (x, y + s),
                    (x - i, y + i),
                    (x - s, y),
                ]
            )


class GlyphMetricPlane(GlyphLevelPlain):
    """Draw left and right margin of glyph on Canvas"""

    pen = get_wxPen(color="gray", style=wx.PENSTYLE_LONG_DASH)

    def __init__(self, parent):
        super().__init__(parent, "GlyphMetric")

    def draw(self, gc):
        gc.SetPen(self.pen)
        for w in (0, self.glyph.width):
            x = round(self.transform.transformPoint((w, 0))[0])
            gc.StrokeLine(x, 0, x, gc.GetSize()[1])


class GlyphGuidePlane(GlyphLevelPlain, GuidelinePlaneMixin):
    """Draw guidelines of glyph on Canvas"""

    selected_pen = get_wxPen(
        color=wx.Colour(255, 0, 0, 64), width=3, style=wx.PENSTYLE_SOLID
    )

    def __init__(self, parent):
        super().__init__(parent, "GlyphGuide")

    def draw(self, gc):
        self.drawGuidelines(gc, self.glyph.guidelines)

    def hitTest(self, x, y):
        return self.hitTestGuidelines(x, y, self.glyph.guidelines)


class GlyphOutlinePlane(GlyphLevelPlain):
    """Draw glyph outline on Canvas"""

    visibleActiveDflt = True
    visibleInactiveDflt = True
    GlyphOutlinePen = wx.BLACK_PEN
    GlyphSelectionPen = wx.RED_PEN
    LabelFont = get_wxFont(pointSize=7, faceName="Small Fonts")

    def __init__(self, parent):
        super().__init__(parent, "GlyphOutline")
        self.hitTestPen = HitTestPen()

    def getInactivePen(self, pen):
        if self.layer.color:
            return get_wxPen(
                self.getInactiveColour(self.layer.color.wx), pen.Width, pen.Style
            )
        return super().getInactivePen(pen)

    def draw(self, gc):
        pen = TransformPen(graphicsRoundingPen, self.transform)
        gc.SetFont(self.LabelFont, wx.LIGHT_GREY)
        for contour in self.glyph:
            if len(contour) == 0:
                continue
            if self.parent.active:
                point = contour[0]
                x, y = self.transform.transformPoint((point.x, point.y))
                # draw contour index
                gc.DrawText(str(self.glyph.contourIndex(contour)), x - 10, y - 15)
                # draw direction indicator
                d = 4
                directionIndicator = gc.CreatePath()
                directionIndicator.MoveToPoint(0, 0)
                directionIndicator.AddCurveToPoint(0, 2 * d, 0, 2 * d, 2 * d, 2 * d)
                directionIndicator.MoveToPoint(d, d)
                directionIndicator.AddLineToPoint(2 * d, 2 * d)
                directionIndicator.AddLineToPoint(d, 3 * d)
                matrix = gc.CreateMatrix()
                matrix.Translate(x, y)
                try:
                    angle = atan2(-(contour[1].y - point.y), contour[1].x - point.x)
                except IndexError:
                    angle = 0
                matrix.Rotate(angle)
                directionIndicator.Transform(matrix)
                gc.SetPen(wx.GREY_PEN)
                gc.StrokePath(directionIndicator)
            for segmentIndex, segment in enumerate(contour.segments):
                segmentType = segment[-1].segmentType
                if segmentType != "move":
                    graphicsRoundingPen.path = gc.CreatePath()
                    if segmentIndex == 0:
                        start = contour.segments[-1][-1]
                    else:
                        start = contour.segments[segmentIndex - 1][-1]
                    pen.moveTo(start)
                    if segmentType == "line":
                        pen.lineTo(segment[0])
                    elif segmentType == "curve":
                        pen.curveTo(*segment)
                    elif segmentType == "qcurve":
                        pen.qCurveTo(*segment)
                    else:
                        log.error("unhandled segment type: %r", segmentType)
                    pen.endPath()
                    if self.parent.active:
                        segmentSelected = start.selected
                        for point in segment:
                            segmentSelected &= point.selected
                        if segmentSelected:
                            gc.SetPen(self.GlyphSelectionPen)
                        else:
                            gc.SetPen(self.GlyphOutlinePen)
                    else:
                        gc.SetPen(self.getInactivePen(self.GlyphOutlinePen))
                    gc.StrokePath(graphicsRoundingPen.path)

    def hitTest(self, x, y):
        if self.glyph:
            self.hitTestPen.reset()
            self.hitTestPen.position = (x, y)
            self.hitTestPen.tolerance = self.canvas.screenToCanvasXrel(3)
            self.glyph.draw(self.hitTestPen)
            hit = self.hitTestPen.hit
            if hit is not None:
                return ContourHit(self.glyph, *hit)


class GlyphComponentPlain(GlyphLevelPlain):
    """
    Draw glyph components on Canvas
    """

    visibleActiveDflt = True
    visibleInactiveDflt = True
    ComponentBrush = get_wxBrush(wx.Colour(192, 192, 192, 200))
    ComponentPen = wx.GREY_PEN
    ComponentSelectionPen = wx.RED_PEN
    LabelFont = get_wxFont(pointSize=7, faceName="Small Fonts")

    def __init__(self, parent):
        super().__init__(parent, "GlyphComponent")

    def draw(self, gc):
        gc.SetBrush(self.ComponentBrush)
        gc.SetFont(self.LabelFont, wx.BLACK)
        graphicsRoundingPen.glyphSet = self.layer
        pen = TransformPen(graphicsRoundingPen, self.transform)
        for componentIndex, component in enumerate(self.glyph.components):
            graphicsRoundingPen.path = gc.CreatePath()
            component.draw(pen)
            if component.selected:
                gc.SetPen(self.ComponentSelectionPen)
            else:
                gc.SetPen(self.ComponentPen)
            gc.DrawPath(graphicsRoundingPen.path, wx.WINDING_RULE)
            point = graphicsRoundingPen.path.Box.LeftBottom
            gc.DrawText(str(componentIndex), point.x, point.y)

    def hitTest(self, x, y):
        for component in self.glyph.components:
            if component.pointInside((x, y)):
                return component


class OnCurvePointsPlain(ShapeDrawPlain):
    """
    Draw labels for On-Curve Points
    """

    Start = {
        "brush": cfgBrush("Start", wx.TRANSPARENT_BRUSH),
        "pen": cfgPen("Start", wx.BLUE_PEN),
        "shape": cfgShape("Start", NODESHAPE_CIRCLE),
        "size": cfgSize("Start", 4),
    }

    Move = {
        "brush": cfgBrush("Move", wx.BLUE_BRUSH),
        "pen": cfgPen("Move", wx.GREY_PEN),
        "shape": cfgShape("Move", NODESHAPE_SQUARE),
        "size": cfgSize("Move", 3),
    }

    Line = {
        "brush": cfgBrush("Line", wx.RED_BRUSH),
        "pen": cfgPen("Line", wx.GREY_PEN),
        "shape": cfgShape("Line", NODESHAPE_SQUARE),
        "size": cfgSize("Line", 3),
    }

    LineSmooth = {
        "brush": cfgBrush("LineSmooth", wx.RED_BRUSH),
        "pen": cfgPen("LineSmooth", wx.GREY_PEN),
        "shape": cfgShape("LineSmooth", NODESHAPE_TRIANGLE),
        "size": cfgSize("LineSmooth", 3),
    }

    PScurve = {
        "brush": cfgBrush("PScurve", wx.GREEN_BRUSH),
        "pen": cfgPen("PScurve", wx.GREY_PEN),
        "shape": cfgShape("PScurve", NODESHAPE_DIAMOND),
        "size": cfgSize("PScurve", 3),
    }

    PScurveSmooth = {
        "brush": cfgBrush("PScurveSmooth", wx.GREEN_BRUSH),
        "pen": cfgPen("PScurveSmooth", wx.GREY_PEN),
        "shape": cfgShape("PScurveSmooth", NODESHAPE_CIRCLE),
        "size": cfgSize("PScurveSmooth", 3),
    }

    TTcurve = {
        "brush": cfgBrush("TTcurve", wx.CYAN_BRUSH),
        "pen": cfgPen("TTcurve", wx.GREY_PEN),
        "shape": cfgShape("TTcurve", NODESHAPE_SQUARE),
        "size": cfgSize("TTcurve", 3),
    }

    def __init__(self, parent):
        super().__init__(parent, "OnCurvePoints")

    def draw(self, gc):
        for contour in self.glyph:
            for point in contour.onCurvePoints:
                if point.segmentType == "move":
                    node = self.Move
                elif point.segmentType == "line":
                    if point.smooth:
                        node = self.LineSmooth
                    else:
                        node = self.Line
                elif point.segmentType == "curve":
                    if point.smooth:
                        node = self.PScurveSmooth
                    else:
                        node = self.PScurve
                elif point.segmentType == "qcurve":
                    node = self.TTcurve
                self.drawShape(gc, point, node)
            if contour.onCurvePoints:
                self.drawShape(gc, contour.onCurvePoints[0], self.Start)

    def hitTest(self, x, y):
        delta = self.canvas.screenToCanvasXrel(4)
        min_x = x - delta
        max_x = x + delta
        min_y = y - delta
        max_y = y + delta
        for contour in self.glyph:
            for point in contour.onCurvePoints:
                if min_x <= point.x <= max_x and min_y <= point.y <= max_y:
                    return point


class OnCurveLabelPlain(LabelPlain):
    """
    Draw labels of on-curve-points on Canvas.
    """

    def __init__(self, parent):
        super().__init__(parent, "OnCurvePointLabels")

    @property
    def points(self):
        result = []
        for contour in self.glyph:
            result.extend(contour.onCurvePoints)
        return result


class OffCurvePointsPlain(ShapeDrawPlain):
    """
    Draw Off-Curve-Points and conecting lines
    """

    TToff = {
        "brush": cfgBrush("TToff", wx.CYAN_BRUSH),
        "pen": cfgPen("TToff", wx.GREY_PEN),
        "shape": cfgShape("TToff", NODESHAPE_CIRCLE),
        "size": cfgSize("TToff", 2),
    }

    PSoff = {
        "brush": cfgBrush("PSoff", wx.YELLOW_BRUSH),
        "pen": cfgPen("PSoff", wx.GREY_PEN),
        "shape": cfgShape("PSoff", NODESHAPE_CIRCLE),
        "size": cfgSize("PSoff", 2),
    }

    ConnectionPen = wx.LIGHT_GREY_PEN

    def __init__(self, parent):
        super().__init__(parent, "OffCurvePoints")

    def draw(self, gc):
        for contour in self.glyph:
            for segmentIndex, segment in enumerate(contour.segments):
                segmentType = segment[-1].segmentType
                if segmentType in ("curve", "qcurve"):
                    if segmentIndex == 0:
                        start = contour.segments[-1][-1]
                    else:
                        start = contour.segments[segmentIndex - 1][-1]
                    x0, y0 = self.transform.transformPoint(start)
                    x0 = round(x0)
                    y0 = round(y0)
                    if segmentType == "curve":
                        node = self.PSoff
                        x1, y1 = self.transform.transformPoint(segment[0])
                        x1 = round(x1)
                        y1 = round(y1)
                        if self.parent.active:
                            if segment[0].selected:
                                gc.SetPen(SelectionPen)
                            else:
                                gc.SetPen(self.ConnectionPen)
                        else:
                            gc.SetPen(self.getInactivePen(self.ConnectionPen))
                        gc.StrokeLine(x0, y0, x1, y1)
                        x2, y2 = self.transform.transformPoint(segment[1])
                        x2 = round(x2)
                        y2 = round(y2)
                        x3, y3 = self.transform.transformPoint(segment[2])
                        x3 = round(x3)
                        y3 = round(y3)
                        if self.parent.active:
                            if segment[1].selected:
                                gc.SetPen(SelectionPen)
                            else:
                                gc.SetPen(self.ConnectionPen)
                        else:
                            gc.SetPen(self.getInactivePen(self.ConnectionPen))
                        gc.StrokeLine(x2, y2, x3, y3)
                        self.drawShape(gc, segment[0], node)
                        self.drawShape(gc, segment[1], node)
                    elif segmentType == "qcurve":
                        node = self.TToff
                        points = []
                        for point in segment:
                            x1, y1 = self.transform.transformPoint(point)
                            x1 = round(x1)
                            y1 = round(y1)
                            gc.StrokeLine(x0, y0, x1, y1)
                            x0 = x1
                            y0 = y1
                            points.append(point)
                        for point in points[:-1]:
                            self.drawShape(gc, point, node)

    def hitTest(self, x, y):
        delta = self.canvas.screenToCanvasXrel(4)
        min_x = x - delta
        max_x = x + delta
        min_y = y - delta
        max_y = y + delta
        for contour in self.glyph:
            for point in [p for p in contour if p.segmentType is None]:
                if min_x <= point.x <= max_x and min_y <= point.y <= max_y:
                    return point


class OFFCurveLabelPlain(LabelPlain):
    """
    Draw labels of off-curve-points on Canvas.
    """

    def __init__(self, parent):
        super().__init__(parent, "OffCurvePointLabels")

    @property
    def points(self):
        result = []
        for contour in self.glyph:
            for point in contour:
                if point.segmentType is None:
                    result.append(point)
        return result


class AnchorsPlane(ShapeDrawPlain):
    """
    Draw Anchors on canvas.
    """

    Anchor = {
        "brush": cfgBrush("Anchor", wx.RED_BRUSH),
        "pen": cfgPen("Anchor", wx.GREY_PEN),
        "shape": cfgShape("Anchor", NODESHAPE_STAR),
        "size": cfgSize("Anchor", 7),
    }

    def __init__(self, parent):
        super().__init__(parent, "Anchors")

    def draw(self, gc):
        for anchor in self.glyph.anchors:
            x, y = self.transform.transformPoint((anchor.x, anchor.y))
            x = round(x)
            y = round(y)
            self.drawShape(gc, anchor, self.Anchor)

    def hitTest(self, x, y):
        delta = self.canvas.screenToCanvasXrel(4)
        min_x = x - delta
        max_x = x + delta
        min_y = y - delta
        max_y = y + delta
        for anchor in self.glyph.anchors:
            if min_x <= anchor.x <= max_x and min_y <= anchor.y <= max_y:
                return anchor


class AnchorLabelPlain(LabelPlain):
    """
    Draw labels of anchors on Canvas.
    """

    visibleActiveDflt = True

    def __init__(self, parent):
        super().__init__(parent, "AnchorLabels")
        self.labelType = self.LT_NAME

    @property
    def points(self):
        return self.glyph.anchors


class GlyphFillPlane(GlyphLevelPlain):
    """
    Draw solid fill of glyph shape.
    """

    visibleActiveDflt = False
    visibleInactiveDflt = False
    lockedDflt = True
    GlyphFillBrush = wx.BLACK_BRUSH
    GlyphFillPen = wx.TRANSPARENT_PEN

    def __init__(self, parent):
        super().__init__(parent, "GlyphFill")

    def draw(self, gc):
        if self.parent.active:
            gc.SetBrush(self.GlyphFillBrush)
            gc.SetPen(self.GlyphFillPen)
        else:
            gc.SetBrush(self.getInactiveBrush(self.GlyphFillBrush))
            gc.SetPen(self.getInactivePen(self.GlyphFillPen))
        graphicsPen.path = gc.CreatePath()
        graphicsPen.glyphSet = self.layer
        pen = TransformPen(graphicsPen, self.transform)
        self.glyph.draw(pen)
        gc.FillPath(graphicsPen.path, wx.WINDING_RULE)


class RedArrowPlane(GlyphLevelPlain):
    """
    Draw Red-Arrow problem indicators
    """

    visibleActiveDflt = False
    visibleInactiveDflt = False
    RedArrow = {
        "brush": cfgBrush("RedArrow", wx.RED_BRUSH),
        "pen": cfgPen("RedArrow", wx.GREY_PEN),
        "shape": cfgShape("RedArrow", NODESHAPE_DIAMOND),
        "size": cfgSize("RedArrow", 18),
    }

    def __init__(self, parent):
        super().__init__(parent, "RedArrows")

    def drawArrow(self, gc, position, errors):
        message = []
        for error in errors:
            if error.vector:
                vector = error.vector
            else:
                vector = (-1, 1)
            angle = atan2(vector[0], -vector[1])
            if error.badness:
                message.append(f"{error.kind} (Severity {error.badness:0.1f})")
            else:
                message.append(f"{error.kind}")
        messageText = ", ".join(message)
        x, y = self.transform.transformPoint((position[0], position[1]))
        size = self.RedArrow["size"]
        head_ratio = 0.6
        w = size * 0.5
        tail_width = 0.2

        gc.SetBrush(self.RedArrow["brush"])
        arrowPath = gc.CreatePath()
        arrowPath.MoveToPoint(0, 0)
        arrowPath.AddLineToPoint(-size * head_ratio, w * 0.5)
        arrowPath.AddLineToPoint(-size * head_ratio, -size * tail_width * 0.5)
        arrowPath.AddLineToPoint(-size, -size * tail_width * 0.5)
        arrowPath.AddLineToPoint(-size, size * tail_width * 0.5)
        arrowPath.AddLineToPoint(-size * head_ratio, size * tail_width * 0.5)
        arrowPath.AddLineToPoint(-size * head_ratio, -w * 0.5)
        arrowPath.CloseSubpath()
        matrix = gc.CreateMatrix()
        matrix.Translate(x, y)
        matrix.Rotate(angle)
        arrowPath.Transform(matrix)
        gc.FillPath(arrowPath)
        # draw Labels
        LabelFont = get_wxFont(pointSize=size * 0.5, faceName="Small Fonts")
        gc.SetFont(LabelFont, wx.LIGHT_GREY)
        w, h = gc.GetTextExtent(messageText)
        text_x, text_y = matrix.TransformPoint(0, 0)
        direction = angle / pi

        if 0 < direction <= 0.4:  # text left
            text_x -= w + 1.2 * size
            text_y -= size
        elif 0.4 < direction <= 0.6:  # text above center
            text_x -= 0.5 * w
            text_y -= h + size
        elif direction > 0.6:  # text right
            text_x += 1.2 * size
            text_y -= size
        elif 0 > direction >= -0.4:  # text left
            text_x -= w + 1.2 * size
            text_y += size
        elif -0.4 > direction >= -0.6:  # text below center
            text_x -= 0.5 * w
            text_y += size
        elif -0.6 > direction:  # right
            text_x += 1.2 * size
            text_y += size

        gc.DrawText(messageText, text_x, text_y)

    def draw(self, gc):
        errors_by_position = {}
        for error in self.glyph.getOutlineErrors():
            if error.position is not None:
                if (error.position[0], error.position[1]) in errors_by_position:
                    errors_by_position[(error.position[0], error.position[1])].extend(
                        [error]
                    )
                else:
                    errors_by_position[(error.position[0], error.position[1])] = [error]
        for position, errors in errors_by_position.items():
            self.drawArrow(gc, position, errors)


class AccentCloudPlane(GlyphLevelPlain):
    """
    Not used yet!
    """

    def __init__(self, parent):
        super(AccentCloudPlane, self).__init__(parent, "AccentCloud")

    def draw(self, gc):
        for anchor in self.glyph.anchors:
            if anchor.name == "top":
                pass


class LayerPlanes(DrawingPlaneStack):
    """
    Collection of Glyph Level Plains
    """

    childType = GlyphLevelPlain
    activeDflt = False
    visibleInactiveDflt = True
    inactiveAlphaDflt = 180

    def __init__(self, parent, name):
        super(LayerPlanes, self).__init__(parent, name)
        self._visibleInactive = self.visibleInactiveDflt
        self.addPlane(GlyphMetricPlane)
        self.addPlane(GlyphGuidePlane)
        self.addPlane(GlyphFillPlane)
        self.addPlane(GlyphComponentPlain)
        self.addPlane(OffCurvePointsPlain)
        self.addPlane(OFFCurveLabelPlain)
        self.addPlane(GlyphOutlinePlane)
        self.addPlane(OnCurvePointsPlain)
        self.addPlane(OnCurveLabelPlain)
        self.addPlane(AnchorsPlane)
        self.addPlane(AnchorLabelPlain)
        self.addPlane(RedArrowPlane)
        self._hitTestOrder = [
            "Anchors",
            "OffCurvePoints",
            "OnCurvePoints",
            "GlyphComponent",
            "GlyphMetric",
            "GlyphGuide",
            "GlyphOutline",
        ]

    @property
    def visible(self):
        return self.active or self._visibleInactive

    @property
    def visibleInactive(self):
        return self._visibleInactive

    @visibleInactive.setter
    def visibleInactive(self, value):
        newValue = bool(value)
        if newValue != self._visibleInactive:
            visible = self.visible
            self._visibleInactive = newValue
            if self.visible != visible:
                self.canvas.Refresh()

    @property
    def layer(self):
        return self.parent.font.layers[self.name]

    @property
    def glyph(self):
        g = self.parent.glyph
        if g is not None:
            glyphName = g.name
            if glyphName in self.layer:
                return self.layer[glyphName]

    def draw(self, gc):
        if self.glyph is not None:
            for plane in [p for p in self._drawingPlanes if p.visible]:
                plane.draw(gc)
        elif self.active:
            canvas = self.canvas
            g = canvas.glyph
            if g is None:
                glyphName = "<None>"
            else:
                glyphName = g.name
            text = f'No glyph "{glyphName}" on layer "{self.name}"'
            cw, ch = canvas.Size
            gc.SetFont(get_wxFont(pointSize=18), wx.BLACK)
            w, h = gc.GetTextExtent(text)
            x = (cw - w) / 2
            y = (ch + h) / 2
            gc.DrawText(text, x, y)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version 3.10.0-4761b0c)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class AssignLayerDialogUI
###########################################################################

class AssignLayerDialogUI ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Assign Layer from Font", pos = wx.DefaultPosition, size = wx.Size( 450,-1 ), style = wx.CAPTION|wx.RESIZE_BORDER )

        self.SetSizeHints( wx.Size( 400,300 ), wx.DefaultSize )

        sizer_main = wx.BoxSizer( wx.VERTICAL )

        sizer_source = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Source" ), wx.VERTICAL )

        sizer_source_grid = wx.FlexGridSizer( 0, 2, 0, 0 )
        sizer_source_grid.AddGrowableCol( 1 )
        sizer_source_grid.SetFlexibleDirection( wx.BOTH )
        sizer_source_grid.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_soure_font = wx.StaticText( sizer_source.GetStaticBox(), wx.ID_ANY, u"Font", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_soure_font.Wrap( -1 )

        sizer_source_grid.Add( self.lbl_soure_font, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_source_fontChoices = []
        self.choice_source_font = wx.Choice( sizer_source.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_source_fontChoices, 0 )
        self.choice_source_font.SetSelection( 0 )
        sizer_source_grid.Add( self.choice_source_font, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_soure_layer = wx.StaticText( sizer_source.GetStaticBox(), wx.ID_ANY, u"Layer", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_soure_layer.Wrap( -1 )

        sizer_source_grid.Add( self.lbl_soure_layer, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_source_layerChoices = []
        self.choice_source_layer = wx.Choice( sizer_source.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_source_layerChoices, 0 )
        self.choice_source_layer.SetSelection( 0 )
        sizer_source_grid.Add( self.choice_source_layer, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizer_source.Add( sizer_source_grid, 1, wx.EXPAND, 5 )


        sizer_main.Add( sizer_source, 0, wx.ALL|wx.EXPAND, 5 )

        sizer_target = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Target" ), wx.VERTICAL )

        sizer_target_grid = wx.FlexGridSizer( 0, 2, 0, 0 )
        sizer_target_grid.AddGrowableCol( 1 )
        sizer_target_grid.SetFlexibleDirection( wx.BOTH )
        sizer_target_grid.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )

        self.lbl_target_font = wx.StaticText( sizer_target.GetStaticBox(), wx.ID_ANY, u"Font", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_target_font.Wrap( -1 )

        sizer_target_grid.Add( self.lbl_target_font, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_target_fontChoices = []
        self.choice_target_font = wx.Choice( sizer_target.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_target_fontChoices, 0 )
        self.choice_target_font.SetSelection( 0 )
        sizer_target_grid.Add( self.choice_target_font, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )

        self.lbl_target_layer = wx.StaticText( sizer_target.GetStaticBox(), wx.ID_ANY, u"Layer", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_target_layer.Wrap( -1 )

        sizer_target_grid.Add( self.lbl_target_layer, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )

        choice_target_layerChoices = []
        self.choice_target_layer = wx.Choice( sizer_target.GetStaticBox(), wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, choice_target_layerChoices, 0 )
        self.choice_target_layer.SetSelection( 0 )
        sizer_target_grid.Add( self.choice_target_layer, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )


        sizer_target_grid.Add( ( 0, 0), 1, wx.EXPAND, 5 )

        self.checkBox_new_glyphs = wx.CheckBox( sizer_target.GetStaticBox(), wx.ID_ANY, u"Create new Glyphs", wx.DefaultPosition, wx.DefaultSize, 0 )
        sizer_target_grid.Add( self.checkBox_new_glyphs, 0, wx.ALL, 5 )


        sizer_target.Add( sizer_target_grid, 1, wx.EXPAND, 5 )


        sizer_main.Add( sizer_target, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_staticline1 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        sizer_main.Add( self.m_staticline1, 0, wx.EXPAND, 0 )

        button_sizer = wx.StdDialogButtonSizer()
        self.button_sizerOK = wx.Button( self, wx.ID_OK )
        button_sizer.AddButton( self.button_sizerOK )
        self.button_sizerCancel = wx.Button( self, wx.ID_CANCEL )
        button_sizer.AddButton( self.button_sizerCancel )
        button_sizer.Realize();

        sizer_main.Add( button_sizer, 1, wx.ALIGN_RIGHT|wx.ALL, 5 )


        self.SetSizer( sizer_main )
        self.Layout()

        self.Centre( wx.BOTH )

        # Connect Events
        self.choice_source_font.Bind( wx.EVT_CHOICE, self.on_choice_source_font )
        self.choice_source_layer.Bind( wx.EVT_CHOICE, self.on_choice_source_layer )
        self.choice_target_font.Bind( wx.EVT_CHOICE, self.on_choice_target_font )
        self.choice_target_layer.Bind( wx.EVT_CHOICE, self.on_choice_target_layer )

    def __del__( self ):
        pass


    # Virtual event handlers, override them in your derived class
    def on_choice_source_font( self, event ):
        event.Skip()

    def on_choice_source_layer( self, event ):
        event.Skip()

    def on_choice_target_font( self, event ):
        event.Skip()

    def on_choice_target_layer( self, event ):
        event.Skip()





import logging
import os
from datetime import datetime

from . import anchor, composite, contour, guideline, metric, misc
from .base import GlyphCommand

log = logging.getLogger(__name__)

glyphCommandRegistry = {}


def registerGlyphCommand(command, group):
    assert issubclass(command, GlyphCommand)
    assert isinstance(group, str)
    if group not in glyphCommandRegistry:
        glyphCommandRegistry[group] = {}
    glyphCommandRegistry[group][command.name] = command


def getCommandClass(name):
    for group in glyphCommandRegistry:
        if name in glyphCommandRegistry[group]:
            return glyphCommandRegistry[group][name]
    return None


def getCommandFromString(string):
    name, paramStr = [s.strip() for s in string.strip().split(":", 1)]
    commandClass = getCommandClass(name)
    if commandClass and issubclass(commandClass, GlyphCommand):
        return commandClass.fromString(paramStr)
    return None


def loadCommandList(filename):
    commandList = []
    if os.path.isfile(filename):
        with open(filename, "r", encoding="utf-8") as commandListFile:
            for line in commandListFile:
                line = line.strip()
                if line and not line.startswith("#"):
                    if "#" in line:
                        line = line.split("#", 1)[0]
                    try:
                        command = getCommandFromString(line)
                    except:
                        log.error('Can not read command: "%s"', line)
                        command = None
                    if isinstance(command, GlyphCommand):
                        commandList.append(command)
    return commandList


def saveCommandList(commandList, filename):
    folder = os.path.dirname(filename)
    if not os.path.isdir(folder):
        os.makedirs(folder)
    with open(filename, "w", encoding="utf-8") as commandListFile:
        commandListFile.write("# UFO WorkBench Command List\n")
        commandListFile.write(f'# saved: {datetime.now().isoformat(" ")}\n\n')
        for command in commandList:
            if isinstance(command, GlyphCommand):
                commandListFile.write(str(command) + "\n")


def executeCommandList(commandList, glyphs):
    fonts = set()
    for glyph in glyphs:
        glyph.holdNotifications()
        for command in commandList:
            command.execute(glyph)
        glyph.destroyRepresentation("outlineErrors")
        glyph.postNotification("Glyph.Changed")
        glyph.releaseHeldNotifications()
        fonts.add(glyph.font)
    for font in fonts:
        font.document.UpdateAllViews(sender=None, hint=None)


# register all available commands
for module in (contour, composite, metric, anchor, guideline, misc):
    group = module.__name__.rsplit(".", 1)[-1].title()
    for name in dir(module):
        obj = getattr(module, name)
        if (
            obj is not GlyphCommand
            and isinstance(obj, type)
            and issubclass(obj, GlyphCommand)
        ):
            registerGlyphCommand(obj, group)



# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Oct 26 2018)
## http://www.wxformbuilder.org/
##
## PLEASE DO *NOT* EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc

###########################################################################
## Class PanelItemPickerUI
###########################################################################

class PanelItemPickerUI ( wx.Panel ):

    def __init__( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.TAB_TRAVERSAL, name = wx.EmptyString ):
        wx.Panel.__init__ ( self, parent, id = id, pos = pos, size = size, style = style, name = name )

        self.SetMinSize( wx.Size( 300,220 ) )

        sizer_main = wx.BoxSizer( wx.HORIZONTAL )

        sizer_options = wx.BoxSizer( wx.VERTICAL )

        self.label_options = wx.StaticText( self, wx.ID_ANY, u"Options", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_options.Wrap( -1 )

        sizer_options.Add( self.label_options, 0, wx.ALL|wx.EXPAND, 5 )

        listBox_optionsChoices = []
        self.listBox_options = wx.ListBox( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, listBox_optionsChoices, wx.LB_EXTENDED )
        sizer_options.Add( self.listBox_options, 1, wx.ALL|wx.EXPAND, 5 )


        sizer_main.Add( sizer_options, 1, wx.EXPAND, 5 )

        sizer_buttons = wx.BoxSizer( wx.VERTICAL )


        sizer_buttons.Add( ( 0, 10), 1, wx.EXPAND, 5 )

        self.button_select = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_select.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_FORWARD, wx.ART_BUTTON ) )
        sizer_buttons.Add( self.button_select, 0, wx.ALL, 5 )

        self.button_unselect = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_unselect.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_BACK, wx.ART_BUTTON ) )
        sizer_buttons.Add( self.button_unselect, 0, wx.ALL, 5 )

        self.button_move_up = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_move_up.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_UP, wx.ART_BUTTON ) )
        sizer_buttons.Add( self.button_move_up, 0, wx.ALL, 5 )

        self.button_move_down = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_move_down.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_GO_DOWN, wx.ART_BUTTON ) )
        sizer_buttons.Add( self.button_move_down, 0, wx.ALL, 5 )

        self.button_clear = wx.BitmapButton( self, wx.ID_ANY, wx.NullBitmap, wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|0 )

        self.button_clear.SetBitmap( wx.ArtProvider.GetBitmap( wx.ART_CLOSE, wx.ART_BUTTON ) )
        sizer_buttons.Add( self.button_clear, 0, wx.ALL, 5 )


        sizer_buttons.Add( ( 0, 0), 1, wx.EXPAND, 5 )


        sizer_main.Add( sizer_buttons, 0, wx.EXPAND, 5 )

        sizer_selection = wx.BoxSizer( wx.VERTICAL )

        self.label_selection = wx.StaticText( self, wx.ID_ANY, u"Selection", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.label_selection.Wrap( -1 )

        sizer_selection.Add( self.label_selection, 0, wx.ALL|wx.EXPAND, 5 )

        listBox_selectionChoices = []
        self.listBox_selection = wx.ListBox( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, listBox_selectionChoices, wx.LB_EXTENDED )
        sizer_selection.Add( self.listBox_selection, 1, wx.ALL|wx.EXPAND, 5 )


        sizer_main.Add( sizer_selection, 1, wx.EXPAND, 5 )


        self.SetSizer( sizer_main )
        self.Layout()
        sizer_main.Fit( self )

        # Connect Events
        self.listBox_options.Bind( wx.EVT_LISTBOX_DCLICK, self.on_listBox_dblclick )
        self.button_select.Bind( wx.EVT_BUTTON, self.on_select )
        self.button_select.Bind( wx.EVT_UPDATE_UI, self.update_button_select )
        self.button_unselect.Bind( wx.EVT_BUTTON, self.on_unselect )
        self.button_unselect.Bind( wx.EVT_UPDATE_UI, self.update_button_unselect )
        self.button_move_up.Bind( wx.EVT_BUTTON, self.on_button_move_up )
        self.button_move_up.Bind( wx.EVT_UPDATE_UI, self.update_button_move_up )
        self.button_move_down.Bind( wx.EVT_BUTTON, self.on_button_move_down )
        self.button_move_down.Bind( wx.EVT_UPDATE_UI, self.update_button_move_down )
        self.button_clear.Bind( wx.EVT_BUTTON, self.on_button_clear )
        self.button_clear.Bind( wx.EVT_UPDATE_UI, self.update_button_clear )
        self.listBox_selection.Bind( wx.EVT_LISTBOX_DCLICK, self.on_listBox_dblclick )

    def __del__( self ):
        pass


    # Virtual event handlers, overide them in your derived class
    def on_listBox_dblclick( self, event ):
        event.Skip()

    def on_select( self, event ):
        event.Skip()

    def update_button_select( self, event ):
        event.Skip()

    def on_unselect( self, event ):
        event.Skip()

    def update_button_unselect( self, event ):
        event.Skip()

    def on_button_move_up( self, event ):
        event.Skip()

    def update_button_move_up( self, event ):
        event.Skip()

    def on_button_move_down( self, event ):
        event.Skip()

    def update_button_move_down( self, event ):
        event.Skip()

    def on_button_clear( self, event ):
        event.Skip()

    def update_button_clear( self, event ):
        event.Skip()






import logging

import wx

from wbDefcon import Font
from wbBase.document import dbg
from wbBase.document.notebook import DocumentPageMixin
from wbBase.document.view import View

# from .feature import FeaturePage
from .glyphGrid import GlyphGridPage
# from .groups import GroupsPage
# from .info import InfoPage
# from .kerning import KerningPage

log = logging.getLogger(__name__)


class UfoFontWindow(wx.Notebook, DocumentPageMixin):
    """
    The main Document Window for UFO Documents
    """

    def __init__(self, parent, doc, view):
        ID = wx.ID_ANY
        pos = wx.DefaultPosition
        size = wx.DefaultSize
        wx.Notebook.__init__(
            self,
            parent,
            ID,
            pos,
            size,
            style=wx.NB_FIXEDWIDTH | wx.NO_BORDER,
            name="UfoFontWindow",
        )
        DocumentPageMixin.__init__(self, doc, view)
        self.SetExtraStyle(wx.WS_EX_VALIDATE_RECURSIVELY)
        self._font = None
        notebookIndex = 0
        tabIcons = wx.ImageList(16, 16)
        self.AssignImageList(tabIcons)

        # GlyphGrid page
        self.glyphGridPage = GlyphGridPage(self)
        self.glyphGridPanel = self.glyphGridPage.glyphGridPanel
        self.AddPage(self.glyphGridPage, "Glyphs", True)
        icon = wx.ArtProvider.GetBitmap("VIEW_ICON", wx.ART_FRAME_ICON)
        if icon.IsOk():
            tabIcons.Add(icon)
            self.SetPageImage(notebookIndex, notebookIndex)
            notebookIndex += 1

        # # Info page
        # self.infoPage = InfoPage(self)
        # self.AddPage(self.infoPage, "Info", False)
        # icon = wx.ArtProvider.GetBitmap("VIEW_INFO", wx.ART_FRAME_ICON)
        # if icon.IsOk():
        #     tabIcons.Add(icon)
        #     self.SetPageImage(notebookIndex, notebookIndex)
        #     notebookIndex += 1

        # # Feature page
        # self.featurePage = FeaturePage(self)
        # self.AddPage(self.featurePage, "Feature", False)
        # icon = wx.ArtProvider.GetBitmap("BLANK", wx.ART_FRAME_ICON)
        # if icon.IsOk():
        #     tabIcons.Add(icon)
        #     self.SetPageImage(notebookIndex, notebookIndex)
        #     notebookIndex += 1

        # # Groups page
        # self.groupsPage = GroupsPage(self)
        # self.AddPage(self.groupsPage, "Groups", False)
        # icon = wx.ArtProvider.GetBitmap("VIEW_GROUP", wx.ART_FRAME_ICON)
        # if icon.IsOk():
        #     tabIcons.Add(icon)
        #     self.SetPageImage(notebookIndex, notebookIndex)
        #     notebookIndex += 1

        # # Kerning page
        # self.kerningPage = KerningPage(self)
        # self.AddPage(self.kerningPage, "Kerning", False)
        # icon = wx.ArtProvider.GetBitmap("VIEW_KERNING", wx.ART_FRAME_ICON)
        # if icon.IsOk():
        #     tabIcons.Add(icon)
        #     self.SetPageImage(notebookIndex, notebookIndex)
        #     notebookIndex += 1

        # # Connect Events
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGING, self.on_PAGE_CHANGING)
        #  Window
        self.Bind(wx.EVT_SET_FOCUS, self.on_SET_FOCUS)

    def __repr__(self):
        return f"<{self.__class__.__name__} of {self._font}>"

    # -------------------------------------------------------------------------
    # properties
    # -------------------------------------------------------------------------

    @property
    def font(self):
        """The wbDefcon Font object of this UfoFontWindow"""
        return self._font

    @font.setter
    def font(self, value):
        assert self._font is None
        assert isinstance(
            value, Font
        ), f"Expected Font, got {value}, type {type(value)}"
        assert value == self._document.font
        self._font = value
        log.debug("Font %r set for %r", self._font, self)
        for page in self.Children:
            page.font = value

    @font.deleter
    def font(self):
        for page in self.Children:
            del page.font

    def Destroy(self):
        del self.font
        return super().Destroy()

    # -------------------------------------------------------------------------
    # event handler
    # -------------------------------------------------------------------------

    def on_PAGE_CHANGING(self, event):
        page = self.GetPage(event.OldSelection)
        if page.Validate():
            if not page.TransferDataFromWindow():
                log.debug("page %r TransferDataFromWindow failed", page)
            event.Skip()
        else:
            log.debug("page %r Validate failed", page)
            event.Veto()

    def on_SET_FOCUS(self, event):
        self.CurrentPage.SetFocus()
        event.Skip()


class UfoFontView(View):
    frameType = UfoFontWindow

    def __repr__(self):
        if self._document:
            return '<%s of "%s">' % (
                self.__class__.__name__,
                self._document.printableName,
            )
        return "<%s of %s>" % (
            self.__class__.__name__,
            self._document.__class__.__name__,
        )

    @property
    def font(self):
        return self.document.font

    def OnCreate(self, doc, flags):
        dbg(f'UfoFontView.OnCreate(doc="{doc}", flags={flags})', indent=1)
        self._document = doc
        self._typeName = doc.template.viewTypeName
        self._frame:UfoFontWindow = self.frameType(self.documentNotebook, doc, self)
        self.documentNotebook.AddPage(
            self._frame,
            doc.printableName,
            True,
            wx.ArtProvider.GetBitmap("F", wx.ART_FRAME_ICON),
        )
        dbg(indent=0)
        return True

    def OnUpdate(self, sender, hint):
        dbg(f"UfoFontView.OnUpdate(sender={sender}, hint={hint})", indent=1)
        super().OnUpdate(sender, hint)
        if hint and hint[0] == "font loaded":
            frame = self._frame
            if frame.font is None:
                frame.font = self.document.font
            frame.title = self._document.title
            if frame and hasattr(frame, "OnTitleIsModified"):
                frame.OnTitleIsModified()
            frame.TransferDataToWindow()
            frame.glyphGridPanel.SendSizeEvent()
        dbg(indent=0)

    def OnClose(self, deleteWindow=True):
        dbg(f"UfoFontView.OnClose(deleteWindow={deleteWindow})", indent=1)
        result = False
        for view in self._document.views:
            if view is not self:
                dbg(f"UfoFontView.OnClose() closing child view {view}")
                if not view.Close(deleteWindow=True):
                    dbg(
                        f"UfoFontView.OnClose() returns {result}, can not close {view}",
                        indent=0,
                    )
                    return result
                dbg("UfoFontView.OnClose() closing child view done")
        result = super().OnClose(deleteWindow=deleteWindow)
        dbg(indent=0)
        return result
